function [holdmu0, holdsigmasq, holda, holdM, fpred] = CCV(data, pred, aprior, Mprior, burnin, numbofits, every)

if ( size(data, 2) == 1 )
    data = data';
end

mu0 = mean(data);
a = 0.5;
sigmasq = 1;
M = 4.5;

n = length(data);
s = 1:n;
mu = mu0 + sqrt((1 - a) * sigmasq) * randn(1, n);
numb = ones(1, n);

holda = zeros(1, numbofits);
holdmu0 = zeros(1, numbofits);
holdsigmasq = zeros(1, numbofits);
holdM = zeros(1, numbofits);
fpred = zeros(1, length(pred));

Maccept = 0;
Mcount = 0;
aaccept = 0;
acount = 0;

for it = 1:(burnin+every*numbofits)

    if ( mod(it, 100) == 0 )
        disp(['iteration = ' num2str(it)]);
        disp(['M accept = ' num2str(Maccept/Mcount)])
        if ( acount > 0 )
            disp(['a accept = ' num2str(aaccept/acount)])
        end
        disp(' ')
    end

    % Update allocations
    for i = 1:n
        numb(s(i)) = numb(s(i)) - 1;

        if ( numb(s(i)) == 0 )
            numb = [numb(1:(s(i)-1)) numb((s(i)+1):length(numb))];
            mu = [mu(1:(s(i)-1)) mu((s(i)+1):length(mu))];
            s = s - (s > s(i));
        end

        logprob = zeros(1, length(numb) + 1);

        logprob(1:length(numb)) = log(numb) - 0.5 * (data(i) - mu).^2 ./ (a * sigmasq) - 0.5 * log(a * sigmasq);
        logprob(end) = log(M) - 0.5 * (data(i) - mu0).^2 ./ sigmasq - 0.5 * log(sigmasq);
        
        prob = exp(logprob - max(logprob));

        fprob = cumsum(prob);
        fprob = fprob ./ fprob(end);

        u = rand;
        s(i) = 1;
        while ( u > fprob(s(i)) )
            s(i) = s(i) + 1;
        end

        if ( s(i) > length(numb) )
            varstar = sigmasq / (1 / (1 - a) + 1 / a);
            mustar = (data(i) / a + mu0 / (1 - a)) / (1 / (1 - a) + 1 / a);
            newmu = mustar + sqrt(varstar) * randn;

            mu = [mu newmu];
            numb = [numb 1];
        else
            numb(s(i)) = numb(s(i)) + 1;
        end
    end


    % Update mu
    for j = 1:length(mu)
        mustar = (sum(data(s==j)) / a + mu0 / (1 - a)) / (numb(j) / a + 1 / (1 - a));
        varstar = sigmasq / (numb(j) / a + 1 / (1 - a));

        mu(j) = mustar + sqrt(varstar) * randn;
    end


    % Update mu0
    meanstar = mean(mu);
    varstar = sigmasq * (1 - a) / length(mu);

    mu0 = meanstar + sqrt(varstar) * randn;


    %Update sigmasq
    alphastar = 0.5 * (n + length(numb));
    betastar = 0.5 * (sum((data - mu(s)).^2) / a + sum((mu - mu0).^2) / (1 - a));

    sigmasq = 1 / gamrnd(alphastar, 1 / betastar);


    % Update a
    alphastar = aprior(1) - 0.5 * n;
    betastar = aprior(2) - 0.5 * length(numb);
                        
    if ( alphastar + betastar < 0 )
        a = Updatea(aprior(1)-0.5*n, aprior(2)-0.5*length(numb), sum((data - mu(s)).^2) / sigmasq, sum((mu - mu0).^2) / sigmasq);
    else
        trans = log(a) - log(1 - a);
        newtrans = trans + 0.9 * randn;
        newa = exp(newtrans) / (1 + exp(newtrans));

        loglike = (aprior(1) - 0.5 * n - 1) * log(a) + (aprior(2) - 0.5 * length(numb) - 1) * log(1 - a);
        loglike = loglike - 0.5 * sum((mu-mu0).^2) / (sigmasq * (1 - a)) - 0.5 * sum((data - mu(s)).^2) / (sigmasq * a);

        newloglike = (aprior(1) - 0.5 * n - 1) * log(newa) + (aprior(2) - 0.5 * length(numb) - 1) * log(1 - newa);
        newloglike = newloglike - 0.5 * sum((mu-mu0).^2) / (sigmasq * (1 - newa)) - 0.5 * sum((data - mu(s)).^2) / (sigmasq * newa);

        logaccept = newloglike - loglike + log(1 / a + 1 / (1 - a)) - log(1 / newa + 1 / (1 - newa));

        accept = 1;
        if (logaccept < 0 )
            accept = exp(logaccept);
        end

        aaccept = aaccept + accept;
        acount = acount + 1;

        if ( rand < accept )
            a = newa;
        end
    end

    % Update M
    newM = M * exp(randn);

    logaccept = gammaln(newM) - gammaln(newM + n) + length(mu) * log(newM);
    logaccept = logaccept - gammaln(M) + gammaln(M + n) - length(mu) * log(M);
    logaccept = logaccept + Mprior(1) * (log(newM) - log(M)) - 2 * Mprior(1) * (log(newM + Mprior(2)) - log(M + Mprior(2)));

    accept = 1;
    if ( logaccept < 0 )
        accept = exp(logaccept);
    end

    if ( rand < accept )
        M = newM;
    end

    Maccept = Maccept + accept;
    Mcount = Mcount + 1;


    % Record values from chain and calculate predictive
    if ( (it > burnin) && (mod(it - burnin, every) == 0) )
        for i = 1:length(pred)
            fpred(i) = fpred(i) + sum(numb./(n+M).*exp(-0.5*(pred(i)-mu).^2./(a*sigmasq))./sqrt(2*pi*a*sigmasq));
            fpred(i) = fpred(i) + M / (n + M) * sum(exp(-0.5*(pred(i)-mu0).^2./sigmasq)/sqrt(2*pi*sigmasq));
        end

        holda((it - burnin) / every) = a;
        holdmu0((it - burnin) / every) = mu0;
        holdsigmasq((it - burnin) / every) = sigmasq;
        holdM((it - burnin) / every) = M;
    end
end

fpred = fpred / numbofits;
