function [holdmu0, holdsigmasq, holda, holdM, fpred] = DCV(data, pred, phi, aprior, Mprior, burnin, numbofits, every)

if ( size(data, 2) == 1 )
    data = data';
end

if ( phi <= 1 )
    'Error phi <= 1'
    exit(1);
end

mu0 = mean(data);
a = 0.5;
sigmasq = 1;
M = 4.5;

n = length(data);
s = 1:n;
numb = ones(1, n);

holda = zeros(1, numbofits);
holdmu0 = zeros(1, numbofits);
holdsigmasq = zeros(1, numbofits);
holdM = zeros(1, numbofits);

mu = mu0 + sqrt((1 - a) * sigmasq) * randn(1, n);
zeta = 1 ./ gamrnd(phi, 1 / (phi - 1), 1, n);

m = 2;

fpred = zeros(1, length(pred));

Maccept = 0;
Mcount = 0;
aaccept = 0;
acount = 0;

for it = 1:(burnin+every*numbofits)

    if ( mod(it, 100) == 0 )
        disp(['iteration = ' num2str(it)])
        disp(['M accept = ' num2str(Maccept/Mcount)])
        if ( acount > 0 )
            disp(['a accept = ' num2str(aaccept/acount)])
        end
        disp(' ')
    end

    % Update allocations
    for i = 1:n

        numb(s(i)) = numb(s(i)) - 1;

        if ( numb(s(i)) == 0 )
            newmu = mu0 + sqrt((1 - a) * sigmasq) * randn(1, m - 1);
            newzeta = 1 ./ gamrnd(phi, 1 / (phi - 1), 1, m - 1);

            logprob = zeros(1, length(numb) + m - 1);

            logprob(1:(s(i)-1)) = log(numb(1:(s(i)-1))) - 0.5 * (data(i) - mu(1:(s(i)-1))).^2 ./ (zeta(1:(s(i)-1)) * a * sigmasq) - 0.5 * log(zeta(1:(s(i)-1)) * a * sigmasq);
            logprob(s(i)) = log(M / m) - 0.5 * (data(i) - mu(s(i)))^2 / (zeta(s(i)) * a * sigmasq) - 0.5 * log(zeta(s(i)) * a * sigmasq);
            logprob((s(i)+1):length(numb)) = log(numb((s(i)+1):end)) - 0.5 * (data(i) - mu((s(i)+1):end)).^2 ./ (zeta((s(i)+1):end) * a * sigmasq) - 0.5 * log(zeta((s(i)+1):end) * a * sigmasq);
            logprob((length(numb)+1):(length(numb)+m-1)) = log(M / m) - 0.5 * (data(i) - newmu).^2 ./ (newzeta * a * sigmasq) - 0.5 * log(newzeta * a * sigmasq);

            prob = exp(logprob - max(logprob));
            
            fprob = cumsum(prob);
            fprob = fprob ./ fprob(end);

            u = rand;
            counter = 1;
            while ( u > fprob(counter) )
                counter = counter + 1;
            end

            if ( counter > length(numb) )
                mu(s(i)) = newmu(counter-length(numb));
                zeta(s(i)) = newzeta(counter-length(numb));
                numb(s(i)) = numb(s(i)) + 1;
            elseif ( s(i) ~= counter )
                numb = [numb(1:(s(i)-1)) numb((s(i)+1):length(numb))];
                mu = [mu(1:(s(i)-1)) mu((s(i)+1):length(mu))];
                zeta = [zeta(1:(s(i)-1)) zeta((s(i)+1):length(zeta))];
                s = s - (s > s(i));

                if ( counter > s(i) )
                    numb(counter - 1) = numb(counter - 1) + 1;
                    s(i) = counter - 1;
                else
                    numb(counter) = numb(counter) + 1;
                    s(i) = counter;
                end
            else
                numb(s(i)) = numb(s(i)) + 1;
            end
        else
            newzeta = 1 ./ gamrnd(phi, 1 / (phi - 1), 1, m);
            newmu = mu0 + sqrt((1 - a) * sigmasq) * randn(1, m);

            logprob = zeros(1, length(numb) + m);

            logprob(1:length(numb)) = log(numb) - 0.5 * (data(i) - mu).^2 ./ (zeta * a * sigmasq) - 0.5 * log(zeta * a * sigmasq);
            logprob((length(numb)+1):(length(numb)+m)) = log(M / m) - 0.5 * (data(i) - newmu).^2 ./ (newzeta * a * sigmasq) - 0.5 * log(newzeta * a * sigmasq);

            prob = exp(logprob - max(logprob));
            
            fprob = cumsum(prob);
            fprob = fprob ./ fprob(end);

            u = rand;
            counter = 1;
            while ( u > fprob(counter) )
                counter = counter + 1;
            end

            if ( counter > length(numb) )
                s(i) = length(numb) + 1;
                mu = [mu newmu(counter-length(numb))];
                zeta = [zeta newzeta(counter-length(numb))];
                numb = [numb 1];
            else
                s(i) = counter;
                numb(s(i)) = numb(s(i)) + 1;
            end
        end
    end

    % Update mu and zeta
    for j = 1:length(mu)
        mustar = (sum(data(s==j)) / (zeta(j) * a) + mu0 / (1 - a)) / (numb(j) / (zeta(j) * a) + 1 / (1 - a));
        varstar = sigmasq / (numb(j) / (zeta(j) * a) + 1 / (1 - a));

        mu(j) = mustar + sqrt(varstar) * randn;

        alphastar = phi + 0.5 * sum(s==j);
        betastar = phi - 1 + 0.5 * sum((data(s==j) - mu(j)).^2) / (a * sigmasq);

        zeta(j) = 1 / gamrnd(alphastar, 1 / betastar);
    end


    % Update mu0
    meanstar = mean(mu);
    varstar = sigmasq * (1 - a) / length(mu);

    mu0 = meanstar + sqrt(varstar) * randn;

    
    % Update sigmasq
    alphastar = 0.5 * (n + length(numb));
    betastar = 0.5 * (sum((data - mu(s)).^2 ./ zeta(s)) / a + sum((mu - mu0).^2) / (1 - a));

    sigmasq = 1 / gamrnd(alphastar, 1 / betastar);

    
    % Update a
    alphastar = aprior(1) - 0.5 * n;
    betastar = aprior(2) - 0.5 * length(numb);
                        
    if ( alphastar + betastar < 0 )
        a = Updatea(aprior(1)-0.5*n, aprior(2)-0.5*length(numb), sum((data - mu(s)).^2 ./ zeta(s)) / sigmasq, sum((mu - mu0).^2) / sigmasq);
    else
        trans = log(a) - log(1 - a);
        newtrans = trans + 0.9 * randn;
        newa = exp(newtrans) / (1 + exp(newtrans));

        loglike = (aprior(1) - 0.5 * n - 1) * log(a) + (aprior(2) - 0.5 * length(numb) - 1) * log(1 - a);
        loglike = loglike - 0.5 * sum((mu-mu0).^2) / (sigmasq * (1 - a)) - 0.5 * sum((data - mu(s)).^2) ./ (zeta(s) * sigmasq * a);

        newloglike = (aprior(1) - 0.5 * n - 1) * log(newa) + (aprior(2) - 0.5 * length(numb) - 1) * log(1 - newa);
        newloglike = newloglike - 0.5 * sum((mu-mu0).^2) / (sigmasq * (1 - newa)) - 0.5 * sum((data - mu(s)).^2) ./ (zeta(s) * sigmasq * newa);

        logaccept = newloglike - loglike + log(1 / a + 1 / (1 - a)) - log(1 / newa + 1 / (1 - newa));

        accept = 1;
        if (logaccept < 0 )
            accept = exp(logaccept);
        end

        aaccept = aaccept + accept;
        acount = acount + 1;

        if ( rand < accept )
            a = newa;
        end
    end


    % Update M
    newM = M * exp(randn);

    logaccept = gammaln(newM) - gammaln(newM + n) + length(mu) * log(newM);
    logaccept = logaccept - gammaln(M) + gammaln(M + n) - length(mu) * log(M);
    logaccept = logaccept + Mprior(1) * (log(newM) - log(M)) - 2 * Mprior(1) * (log(newM + Mprior(2)) - log(M + Mprior(2)));

    accept = 1;
    if ( logaccept < 0 )
        accept = exp(logaccept);
    end

    if ( rand < accept )
        M = newM;
    end

    Maccept = Maccept + accept;
    Mcount = Mcount + 1;

    
    % Record values from chain and calculate predictive
    if ( (it > burnin) && (mod(it - burnin, every) == 0) )
        for i = 1:length(pred)
            fpred(i) = fpred(i) + sum(numb./(n+M).*exp(-0.5*(pred(i)-mu).^2./(zeta*a*sigmasq))./sqrt(2*pi*zeta*a*sigmasq));
            newmu = mu0 + sqrt((1 - a) * sigmasq) * randn(1, m);
            newzeta = 1 ./ gamrnd(phi, 1, 1, m) ./ (phi - 1);
            fpred(i) = fpred(i) + M / ((n + M) * m) * sum(exp(-0.5*(pred(i)-newmu).^2./(newzeta*a*sigmasq))./sqrt(2*pi*newzeta*a*sigmasq));
        end

        holda((it - burnin) / every) = a;
        holdmu0((it - burnin) / every) = mu0;
        holdsigmasq((it - burnin) / every) = sigmasq;
        holdM((it - burnin) / every) = M;
    end
end

fpred = fpred / numbofits;
